;Simple Thermometer with DS18B20 one-wire sensor
;
;CONTROLLER
;  AT90S1200, AT90S2313 or ATTiny2313 @ 4 MHz (ext. XTAL)
;  Fuses for ATTiny2313: low=0xDD, high=0xDB, ext=0xFF
.equ AT90S1200 = 0 ;set to 1 when using AT90S1200, 0 for *2313
;
;DISPLAY
;        1       2       3       4
;   ,---------------------------------,
;   |  ,---,   ,---, o ,---, o ,---,  |
;   |  |   |   |   | 3 |   | 4 |   |  |
;   |  }---{   }---{   }---{   }---{  |
;   |  |   | 1 |   | 2 |   |   |   |  |
;   |  '---' o '---' o '---'   '---'  |
;   '---------------------------------'
;
;PORTS:
;  Display Segments: PORTB
.equ	SEG_A = 0
.equ	SEG_B = 1
.equ	SEG_C = 2
.equ	SEG_D = 3
.equ	SEG_E = 4
.equ	SEG_F = 5
.equ	SEG_G = 6
.equ	SEG_DP = 7
;  Everything else: PORTD
;  (there are more functions than port pins, set unwanted functions to 7)
;  (do not disable PUSHBUTTON or ONEWIRE -- don't set those to 7)
.equ	PUSHBUTTON = 4 ;(active low)
.equ	BUZZER = 5 ;(active low)
.equ	SWITCH = 7 ;(low if temperature too low/high)
.equ	SWITCH_LOW = 7 ;(low if temperature too low)
.equ	SWITCH_HIGH = 7 ;(low if temperature too high)
.equ	ONEWIRE = 6 ;(external pull-up 4.7k, no Vcc for DS18B20 needed)
.equ	ANODE_1 = 0 ;leftmost display (active low)
.equ	ANODE_2 = 1 ;(active low)
.equ	ANODE_3 = 2 ;(active low)
.equ	ANODE_4 = 3 ;rightmost display (active low)

;===============================================================================

.if (AT90S1200 == 1)
.include "../../include/1200def.inc"
.else
.include "../../include/2313def.inc"
.endif

;R0..R15: RAM for moving average filter

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	disp = R19
.def	disp1 = R20
.def	disp2 = R21
.def	disp3 = R22
.def	disp4 = R23
.def	setval = R24
.def	readL = R25
.def	readH = R26
.def	stateL = R27
.def	stateH = R28
.def	key_timer = R29
;R30 = ZL (used for accessing R0~R15 with st/ld)
.def	flags = R31
	.equ	fAlarm = 0
	.equ	fAlarmCanceled = 1
	.equ	fMenu = 2
	.equ	fKeyShort = 3
	.equ	fKeyLong = 4
	.equ	fDispBright = 5
	.equ	fFirstSample = 6

;7-Segment Display Digits (do not modify the numbers!)
.equ	DIGIT_0 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F)
.equ	DIGIT_1 = ~(1<<SEG_B | 1<<SEG_C)
.equ	DIGIT_2 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_D | 1<<SEG_E | 1<<SEG_G)
.equ	DIGIT_3 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_G)
.equ	DIGIT_4 = ~(1<<SEG_B | 1<<SEG_C | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_5 = ~(1<<SEG_A | 1<<SEG_C | 1<<SEG_D | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_6 = ~(1<<SEG_A | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_7 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C)
.equ	DIGIT_8 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_9 = ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_L = ~(1<<SEG_D | 1<<SEG_E | 1<<SEG_F)
.equ	DIGIT_H = ~(1<<SEG_B | 1<<SEG_C | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G)
.equ	DIGIT_MINUS = ~(1<<SEG_G)
.equ	DIGIT_SPACE = ~(0)
.equ	DIGIT_CELSIUS = ~(1<<SEG_A | 1<<SEG_F | 1<<SEG_G | 1<<SEG_DP)

;EEPROM Base Address
.equ	EE_BASE = 23

;===============================================================================

.eseg
.org EE_BASE
	;default temperature min/max alarms (continuous alarm if min > max)
	;examples: '-',1,3 (-13'C), 0,2,0 (20'C), 1,2,3 (123'C)
	.db 0,2,0, 0,2,4 ;min. 20'C, max. 24'C

;===============================================================================

.cseg
.org 0x000
reset:
	.if (AT90S1200 == 0)
	;stackpointer
	ldi	temp, RAMEND
	out	SREG, temp
	.endif
	
	;ports
	ldi	temp, 0xFF
	out	DDRB, temp
	ldi	temp, 1<<ANODE_1 | 1<<ANODE_2 | 1<<ANODE_3 | 1<<ANODE_4 | 1<<BUZZER | 1<<SWITCH | 1<<SWITCH_LOW | 1<<SWITCH_HIGH
	out	DDRD, temp
	ldi	temp, 1<<ANODE_1 | 1<<ANODE_2 | 1<<ANODE_3 | 1<<ANODE_4 | 1<<BUZZER | 1<<PUSHBUTTON | 1<<SWITCH | 1<<SWITCH_LOW | 1<<SWITCH_HIGH
	out	PORTD, temp
	
	;Timer 0 (muxing frequency)
	ldi	temp, 0x03 ;Clk/64
	out	TCCR0, temp
	
	;init registers
	ldi	stateL, 0
	ldi	stateH, 0
	ldi	flags, 1<<fFirstSample
	ldi	key_timer, 0
	
	;init display
	ldi	disp1, DIGIT_MINUS
	ldi	disp2, DIGIT_MINUS & ~(1<<SEG_DP)
	ldi	disp3, DIGIT_MINUS
	ldi	disp4, DIGIT_CELSIUS
	
	;configure onewire sensor
	rcall	onewire_reset
	ldi	readH, 0xCC ;Skip ROM
	rcall	onewire_writebyte
	ldi	readH, 0x4E ;Write Scratchpad
	rcall	onewire_writebyte
	rcall	onewire_writebyte ;readH is 0 after last sendbyte
	rcall	onewire_writebyte ;readH is 0 after last sendbyte
	ldi	readH, 0xC7 ;12 bit resolution
	rcall	onewire_writebyte
	
;-------------------------------------------------------------------------------

loop:
	;multiplex display
	;Display 1
	cpi	disp, 0
	brne	mux_1_end
	sbi	PORTD, ANODE_4
	out	PORTB, disp1
	cbi	PORTD, ANODE_1
mux_1_end:
	;Display 2
	cpi	disp, 1
	brne	mux_2_end
	sbi	PORTD, ANODE_1
	out	PORTB, disp2
	cbi	PORTD, ANODE_2
mux_2_end:
	;Display 3
	cpi	disp, 2
	brne	mux_3_end
	sbi	PORTD, ANODE_2
	out	PORTB, disp3
	cbi	PORTD, ANODE_3
mux_3_end:
	;Display 4
	cpi	disp, 3
	brne	mux_4_end
	sbi	PORTD, ANODE_3
	out	PORTB, disp4
	cbi	PORTD, ANODE_4
	;check pushbutton (only once every 4 mux-loops = ~250 Hz)
	sbic	PIND, PUSHBUTTON
	rjmp	key_released
	cpi	key_timer, 255
	breq	key_end
	inc	key_timer
	cpi	key_timer, 125 ;long keypress: 500 ms
	brne	key_end
	ori	flags, 1<<fKeyLong
	rjmp	key_end
key_released:
	cpi	key_timer, 10 ;short keypress: min. 40 ms
	brlo	key_released_end
	cpi	key_timer, 125 ;a long keypress is not a short keypress ;-)
	brsh	key_released_end
	ori	flags, 1<<fKeyShort
key_released_end:
	clr	key_timer
key_end:
mux_4_end:

	;toggle buzzer output if alarm enabled
	sbrs	flags, fAlarm
	rjmp	mux_buzzer_off ;no alarm
	sbis	PORTD, BUZZER
	rjmp	mux_buzzer_off ;toggle => off
	cbi	PORTD, BUZZER ;toggle => on
	rjmp	mux_buzzer_end
mux_buzzer_off:
	sbi	PORTD, BUZZER
mux_buzzer_end:
	
	;check for keypress
	;short keypress: cancel alarm, menu: increment value
	sbrs	flags, fKeyShort
	rjmp	key_short_end
	sbrc	flags, fAlarm
	ori	flags, 1<<fAlarmCanceled
	andi	flags, ~(1<<fAlarm | 1<<fKeyShort)
	sbrs	flags, fMenu
	rjmp	key_short_end
	inc	setval
	cpi	setval, 10
	brlo	PC+2
	ldi	setval, 0
	sbrc	count, 1 ;0, 4: allow 0, 1, and '-' / 1, 2, 5, 6: allow 0 to 9
	rjmp	key_short_inc_end
	cpi	setval, 2
	brlo	key_short_inc_end
	sbrs	count, 0
	ldi	setval, '-'
key_short_inc_end:
	mov	temp, setval ;update display
	rcall	encode_7seg
	sbrc	count, 1
	rjmp	key_short_disp_3
	sbrs	count, 0
	mov	disp1, temp2
	sbrc	count, 0
	mov	disp2, temp2
	rjmp	key_short_end
key_short_disp_3:
	mov	disp3, temp2
key_short_end:
	;long keypress; menu
	;count => 0 = min 100's, 1 = min 10's, 2 = min 1's, (skip 3)
	;         4 = max 100's, 5 = max 10's, 6 = max 1's
	;         7 = exit menu
	sbrs	flags, fKeyLong
	rjmp	key_long_end
	andi	flags, ~(1<<fKeyLong | 1<<fAlarm) ;cancel alarm when entering menu
	inc	count
	sbrs	flags, fMenu ;enter menu: count keeps track of position in menu
	clr	count
	ori	flags, 1<<fMenu
	cpi	count, 3
	brne	PC+2
	inc	count
	cpi	count, 0 ;min T, 100's
	brne	menu_min_100_end
	ldi	temp2, EE_BASE
	rcall	ee_read_encode
	mov	setval, temp
	mov	disp1, temp2
	ldi	temp2, EE_BASE+1
	rcall	ee_read_encode
	mov	disp2, temp2
	ldi	temp2, EE_BASE+2
	rcall	ee_read_encode
	mov	disp3, temp2
	ldi	disp4, DIGIT_L
menu_min_100_end:
	cpi	count, 1
	brne	menu_min_10_end
	ldi	temp2, EE_BASE
	mov	temp, setval
	rcall	ee_write
	ldi	temp2, EE_BASE+1
	rcall	ee_read
	mov	setval, temp
menu_min_10_end:
	cpi	count, 2
	brne	menu_min_1_end
	ldi	temp2, EE_BASE+1
	mov	temp, setval
	rcall	ee_write
	ldi	temp2, EE_BASE+2
	rcall	ee_read
	mov	setval, temp
menu_min_1_end:
	cpi	count, 4 ;max T, 100's
	brne	menu_max_100_end
	ldi	temp2, EE_BASE+2
	mov	temp, setval
	rcall	ee_write
	ldi	temp2, EE_BASE+3
	rcall	ee_read_encode
	mov	setval, temp
	mov	disp1, temp2
	ldi	temp2, EE_BASE+4
	rcall	ee_read_encode
	mov	disp2, temp2
	ldi	temp2, EE_BASE+5
	rcall	ee_read_encode
	rcall	encode_7seg
	mov	disp3, temp2
	ldi	disp4, DIGIT_H
menu_max_100_end:
	cpi	count, 5
	brne	menu_max_10_end
	ldi	temp2, EE_BASE+3
	mov	temp, setval
	rcall	ee_write
	ldi	temp2, EE_BASE+4
	rcall	ee_read
	mov	setval, temp
menu_max_10_end:
	cpi	count, 6
	brne	menu_max_1_end
	ldi	temp2, EE_BASE+4
	mov	temp, setval
	rcall	ee_write
	ldi	temp2, EE_BASE+5
	rcall	ee_read
	mov	setval, temp
menu_max_1_end:
	cpi	count, 7 ;exit menu
	brne	key_long_end
	ldi	temp2, EE_BASE+5
	mov	temp, setval
	rcall	ee_write
	rjmp	reset
key_long_end:
	
	;continue onewire transfer (or wait)
	sbrc	flags, fMenu ;stop all onewire transfers while menu is shown
	rjmp	mux_onewire_end
	cpi	stateH, 0x00
	brne	state_14xx
	;0000: start onewire transaction: send reset
	cpi	stateL, 0x00
	brne	PC+3
	rcall	onewire_reset
	rjmp	mux_onewire_next
	;0001: address onewire device
	cpi	stateL, 0x01
	brne	PC+4
	ldi	readH, 0xCC ;Skip ROM
	rcall	onewire_writebyte
	rjmp	mux_onewire_next
	;0002: issue "convert T" command
	cpi	stateL, 0x02
	brne	PC+5
	ldi	readH, 0x44 ;Convert T
	rcall	onewire_writebyte
	sbi	PORTD, ONEWIRE ;enable strong pullup
	sbi	DDRD, ONEWIRE
	;(fall through to rjmp mux_onewire_next)
	;0003~03FF: delay (Convert T needs 750 ms maximum, this gives it ~1 s)
	rjmp	mux_onewire_next

state_14xx:
	cpi	stateH, 0x14
	brlo	state_4xx
	;1400: end of cycle
	ldi	stateL, 0
	ldi	stateH, 0
	rjmp	mux_onewire_end
	
state_4xx:
	cpi	stateH, 0x04
	breq	PC+2
	rjmp	mux_onewire_next
	;0400: reset onewire device
	cpi	stateL, 0x00
	brne	PC+5
	cbi	DDRD, ONEWIRE ;disable strong pullup
	cbi	PORTD, ONEWIRE
	rcall	onewire_reset
	rjmp	mux_onewire_next
	;0401: issue "Skip ROM" command
	cpi	stateL, 0x01
	brne	PC+4
	ldi	readH, 0xCC ;Skip ROM
	rcall	onewire_writebyte
	rjmp	mux_onewire_next
	;0402: issue "Read Scratchpad" command
	cpi	stateL, 0x02
	brne	PC+4
	ldi	readH, 0xBE ;Read Scratchpad
	rcall	onewire_writebyte
	rjmp	mux_onewire_next
	;0403: read temperature LSB
	cpi	stateL, 0x03
	brne	PC+4
	rcall	onewire_readbyte
	mov	readL, readH
	rjmp	mux_onewire_next
	;0404: read temperature MSB
	cpi	stateL, 0x04
	brne	PC+3
	rcall	onewire_readbyte ;(result already is in readH, no 'mov' necessary)
	rjmp	mux_onewire_next
	;0405: calculate moving average, convert temperature to BCD, update display
	cpi	stateL, 0x05
	breq	PC+2
	;0406~13FF: delay (one sample every ~5s)
	rjmp	mux_onewire_next
	
	;moving average ("window") filter
	sbrs	flags, fFirstSample
	rjmp	moving_average
	;first conversion: fill all moving average storage slots with this value
	andi	flags, ~(1<<fFirstSample)
	ldi	ZL, 2
fill_ram_loop:
	st	Z, readL
	subi	ZL, -1 ;inc
	st	Z, readH
	subi	ZL, -1 ;inc, clears halfcarry when incrementing 15 => 16
	brhs	fill_ram_loop
	rjmp	moving_average_end
moving_average:
	;"move" window filter
	ldi	ZL, 2
moving_average_shift:
	ld	temp, Z
	subi	ZL, 2
	st	Z, temp
	subi	ZL, -3
	ld	temp, Z
	subi	ZL, 2
	st	Z, temp
	subi	ZL, -3 ;clears halfcarry when incrementing 13 => 16
	brhs	moving_average_shift
	mov	R14, readL
	mov	R15, readH
	;calculate average
	ldi	ZL, 0
moving_average_add:
	ld	temp, Z
	inc	ZL
	ld	temp2, Z
	inc	ZL
	add	readL, temp
	adc	readH, temp2
	cpi	ZL, 14
	brlo	moving_average_add
	ldi	temp2, 3 ;divide by 2 three times => division by 8 (no rounding)
moving_average_divide:
	asr	readH
	ror	readL
	dec	temp2
	brne	moving_average_divide
moving_average_end:

	;convert binary value to BCD (absolute value, T flag is sign)
	clt
	sbrs	readH, 7
	rjmp	convert_bcd_abs
	;negative: set T flag and calculate 2's complement: invert, then add 1
	set
	com	readL
	com	readH
	subi	readL, LOW(-1)
	sbci	readH, HIGH(-1)
convert_bcd_abs:
	;disp3 <= 1/10's (map 0..F to 0,1,1,2,3,3,4,4,5,6,6,7,8,8,9,9)
	mov	temp, readL
	andi	temp, 0x0F
	mov	disp3, temp
	inc	disp3
	sbrc	temp, 2
	inc	disp3
	sbrc	temp, 3
	subi	disp3, -2
	lsr	disp3
	;disp1 <= 10's (disp1 >= 10 if temperature >= 100), disp2 <= 1's
	andi	readL, 0xF0
;	andi	readH, 0x0F ;shouldn't be neccessary, just a precaution
	or	readL, readH
	swap	readL
	mov	disp2, readL
	clr	disp1
convert_bcd_abs_loop:
	cpi	disp2, 10
	brlo	convert_bcd_abs_end
	subi	disp2, 10
	inc	disp1
	rjmp	convert_bcd_abs_loop
convert_bcd_abs_end:

	;check if temperature is above/below limit
	ldi	readL, 0
	ldi	temp2, EE_BASE ;min. temp
	rcall	ee_read
	cpi	temp, 0xFF
	brne	PC+2
	rjmp	check_t_end
	ldi	temp2, EE_BASE+1
	cpi	temp, '-'
	breq	check_t_min_neg
	brts	check_t_min_failed
	mov	count, temp
	rcall	ee_read
	sbrc	count, 0
	subi	temp, -10
	cp	disp1, temp
	brlo	check_t_min_failed
	brne	check_t_min_ok
	ldi	temp2, EE_BASE+2
	rcall	ee_read
	cp	disp2, temp
	brlo	check_t_min_failed
	rjmp	check_t_min_ok
check_t_min_neg:
	brtc	check_t_min_ok
	rcall	ee_read
	cp	temp, disp1
	brlo	check_t_min_failed
	brne	check_t_min_ok
	ldi	temp2, EE_BASE+2
	rcall	ee_read
	cp	temp, disp2
	brlo	check_t_min_failed
	brne	check_t_min_ok
	cpi	disp3, 0
	brne	check_t_min_failed
check_t_min_ok:
	sbi	PORTD, SWITCH_LOW
	rjmp	check_t_max
check_t_min_failed:
	ldi	readL, 1
	cbi	PORTD, SWITCH_LOW
check_t_max:
	ldi	temp2, EE_BASE+3 ;max. temp
	rcall	ee_read
	ldi	temp2, EE_BASE+4
	cpi	temp, '-'
	breq	check_t_max_neg
	brts	check_t_max_ok
	mov	count, temp
	rcall	ee_read
	sbrc	count, 0
	subi	temp, -10
	cp	temp, disp1
	brlo	check_t_max_failed
	brne	check_t_max_ok
	ldi	temp2, EE_BASE+5
	rcall	ee_read
	cp	temp, disp2
	brlo	check_t_max_failed
	brne	check_t_max_ok
	cpi	disp3, 0
	brne	check_t_max_failed
	rjmp	check_t_max_ok
check_t_max_neg:
	brtc	check_t_max_failed
	rcall	ee_read
	cp	disp1, temp
	brlo	check_t_max_failed
	brne	check_t_max_ok
	ldi	temp2, EE_BASE+5
	rcall	ee_read
	cp	disp2, temp
	brlo	check_t_max_failed
check_t_max_ok:
	sbi	PORTD, SWITCH_HIGH
	cpi	readL, 0
	brne	check_t_failed
	;temperature within limits: clear alarm
	sbi	PORTD, SWITCH
	andi	flags, ~(1<<fAlarm | 1<<fAlarmCanceled)
	rjmp	check_t_end
check_t_max_failed:
	cbi	PORTD, SWITCH_LOW
check_t_failed:
	;temperature check failed, sound alarm (if not previously canceled)
	cbi	PORTD, SWITCH
	sbrs	flags, fAlarmCanceled
	ori	flags, 1<<fAlarm
check_t_end:
	
	;update display
	cpi	disp1, 10
	brsh	update_disp_nodecimals ;temperature >= 100
	brtc	update_disp_decimals ;temperature positive, but < 100
	cpi	disp1, 0
	brne	update_disp_nodecimals ; temperature <= -10
	ldi	disp1, DIGIT_MINUS ;temperature negative, but > -10
update_disp_decimals:
	;display format: "23.5'C", "9.4'C" or "-4.2'C"
	mov	temp, disp3
	rcall	encode_7seg
	mov	disp3, temp2
	mov	temp, disp2
	rcall	encode_7seg
	mov	disp2, temp2
	andi	disp2, ~(1<<SEG_DP) ;enable dp
	brts	mux_onewire_next ;format: "-4.2'C"
	mov	temp, disp1
	rcall	encode_7seg
	mov	disp1, temp2
	cpi	disp1, DIGIT_0
	brne	PC+2 ;format: '23.5'C"
	ldi	disp1, DIGIT_SPACE ;format: "9.4'C"
	rjmp	mux_onewire_next
update_disp_nodecimals:
	;display format: "105'C" or "-15'C"
	cpi	disp3, 5 ;round up if digit after decimal point >= 5
	brlo	PC+2
	inc	disp2
	cpi	disp2, 10
	brlo	PC+3
	subi	disp2, 10
	inc	disp1
	mov	temp, disp2
	rcall	encode_7seg
	mov	disp3, temp2
	mov	temp, disp1
	ldi	disp1, DIGIT_MINUS ;format: "-15'C"
	brts	PC+3
	ldi	disp1, DIGIT_1 ;format: "105'C"
	subi	temp, 10
	rcall	encode_7seg
	mov	disp2, temp2
	
mux_onewire_next:
	subi	stateL, LOW(-1)
	sbci	stateH, HIGH(-1)
mux_onewire_end:

mux_delay:
	;wait until Timer 0 overflows
	in	temp, TIFR
	sbrs	temp, TOV0
	rjmp	mux_delay
	ldi	temp, 1<<TOV0
	out	TIFR, temp
	ldi	temp, 256-63 ;4 MHz / 64 / 63 = ~1 kHz
	out	TCNT0, temp
	
	sbrs	flags, fMenu
	rjmp	mux_inc
	mov	temp, count
	andi	temp, 0x03
	cp	disp, temp
	brne	mux_inc
	ldi	temp, 1<<fDispBright
	eor	flags, temp
	sbrs	flags, fDispBright
mux_inc:
	inc	disp
	andi	disp, 0x03
	rjmp	loop
	
;===============================================================================
;
; DISPLAY
;
;===============================================================================

encode_7seg:
	;encode BCD ("temp") to 7-segment ("temp2")
	;
	;THEORY:
	;  enable all segments except F, then
	;  a) (5) disable A if temp = 1
	;     (6) disable A if temp = 4
	;  b) (9) disable B if temp = {5, 6}
	;  c) (2) disable C if temp = 2
	;  d) (5) disable D if temp = 1
	;     (6) disable D if temp = 4
	;     (8) disable D if temp = 7
	;  e) (1) disable E if bit 0 of temp is 1
	;     (5) disable E if temp = 1
	;     (6) disable E if temp = 4
	;  f) (4) enable F if temp = 0
	;     (7) enable F if temp >= 4
	;     (8) disable F if temp = 7
	;  g) (3) disable G if temp < 2
	;     (8) disable G if temp = 7
	
	ldi	temp2, ~(1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_G)
	
	sbrc	temp, 0
	ori	temp2, 1<<SEG_E ;(1) bit 0 in temp is 1
	
	cpi	temp, 2
	brne	PC+2
	ori	temp2, 1<<SEG_C ;(2) temp = 2
	brsh	PC+6
	ori	temp2, 1<<SEG_G ;(3) temp < 2
	sbrs	temp, 0
	andi	temp2, ~(1<<SEG_F) ;(4) temp = 0
	sbrc	temp, 0
	ori	temp2, 1<<SEG_A | 1<<SEG_D | 1<<SEG_G ;(5) temp = 1
	
	cpi	temp, 4
	brlo	PC+4
	brne	PC+2
	ori	temp2, 1<<SEG_A | 1<<SEG_D | 1<<SEG_E ;(6) temp = 4
	andi	temp2, ~(1<<SEG_F) ;(7) temp >= 4
	
	cpi	temp, 7
	brne	PC+2
	ori	temp2, 1<<SEG_D | 1<<SEG_F | 1<<SEG_G ;(8) temp = 7
	brsh	PC+4
	cpi	temp, 5
	brlo	PC+2
	ori	temp2, 1<<SEG_B ;(9) temp = 5 or temp = 6
	
	;minus sign: ignore above result and set temp2 directly
	cpi	temp, '-'
	brne	PC+2
	ldi	temp2, DIGIT_MINUS
	
	ret
	
;===============================================================================
;
; ONEWIRE
;
;===============================================================================

onewire_reset:
	;reset all onewire devices
	sbi	DDRD, ONEWIRE
	ldi	temp, 60
	rcall	delay10us
	cbi	DDRD, ONEWIRE
	rcall	delay60us
	sbis	PIND, ONEWIRE
	rjmp	PC-1
	ret
	
;--------------------

onewire_writebyte:
	;send byte in "readH" to onewire bus
	ldi	count, 8
onewire_writebyte_loop:
	sbrc	readH, 0
	rjmp	onewire_writebyte_one
	;zero
	sbi	DDRD, ONEWIRE
	rcall	delay60us
	cbi	DDRD, ONEWIRE
	rjmp	onewire_writebyte_end
onewire_writebyte_one:
	;one
	sbi	DDRD, ONEWIRE
	rcall	delay_7cyc
	cbi	DDRD, ONEWIRE
	rcall	delay60us
onewire_writebyte_end:
	lsr	readH
	dec	count
	brne	onewire_writebyte_loop
	ret
	
;--------------------

onewire_readbyte:
	;read one byte from onewire bus to "readH"
	ldi	count, 8
onewire_readbyte_loop:
	lsr	readH
	sbi	DDRD, ONEWIRE
	rcall	delay_7cyc
	cbi	DDRD, ONEWIRE
	ldi	temp, 1
	rcall	delay10us
	sbic	PIND, ONEWIRE
	ori	readH, 0x80
	rcall	delay60us
	dec	count
	brne	onewire_readbyte_loop
	ret
	
;--------------------

delay60us:
	;60us delay
	ldi	temp, 6
delay10us:
	;temp * 10us delay
	ldi	temp2, 13 ;13 for 4 MHz, 14 for 4.194304 MHz
delay10us_loop:
	dec	temp2
	brne	delay10us_loop
	dec	temp
	brne	delay10us
delay_7cyc:
	;7 cycles delay (just rcall + ret)
	ret

;===============================================================================
;
; EEPROM
;
;===============================================================================

ee_read_encode:
	;read a byte from EEPROM at address "temp2" and encode to 7-segment
	;returns EEPROM data in "temp", 7-segment data in "temp2"
	rcall	ee_read
	rcall	encode_7seg
	ret

;--------------------

ee_read:
	;read a byte from EEPROM
	out	EEAR, temp2
	sbi	EECR, EERE
	in	temp, EEDR
	ret
	
;--------------------

ee_write:
	;write a byte too EEPROM
	out	EEAR, temp2
	out	EEDR, temp
.if (AT90S1200 == 0)
	sbi	EECR, EEMWE
.endif
	sbi	EECR, EEWE
ee_write_wait:
	sbic	EECR, EEWE
	rjmp	ee_write_wait
	ret
